@@ -10,9 +10,9 @@ from numbers import Number
 from ..expr import Reduction, Field, Projection, Broadcast, Selection, ndim
 from ..expr import Distinct, Sort, Head, Label, ReLabel, Expr, Slice, Join
 from ..expr import std, var, count, nunique, Summary, IsIn
-from ..expr import BinOp, UnaryOp, USub, Not, nelements
+from ..expr import BinOp, UnaryOp, USub, Not, nelements, Repeat, Concat, Interp
 from ..expr import UTCFromTimestamp, DateTimeTruncate
-from ..expr import Transpose, TensorDot
+from ..expr import Transpose, TensorDot, Coerce
 from ..utils import keywords
 
 from .core import base, compute
@@ -45,7 +45,7 @@ try:
     from .numba import broadcast_numba as broadcast_ndarray
 except ImportError:
     def broadcast_ndarray(t, *data, **kwargs):
-        scope = kwargs.pop('scope')
+        del kwargs['scope']
         d = dict(zip(t._scalar_expr._leaves(), data))
         return compute(t._scalar_expr, d, **kwargs)
 
@@ -55,6 +55,46 @@ for i in range(2, 6):
     compute_up.register(Broadcast, *([(np.ndarray, Number)] * i))(broadcast_ndarray)
 
 
+@dispatch(Repeat, np.ndarray)
+def compute_up(t, data, _char_mul=np.char.multiply, **kwargs):
+    if isinstance(t.lhs, Expr):
+        return _char_mul(data, t.rhs)
+    else:
+        return _char_mul(t.lhs, data)
+
+
+@compute_up.register(Repeat, np.ndarray, (np.ndarray, base))
+@compute_up.register(Repeat, base, np.ndarray)
+def compute_up_np_repeat(t, lhs, rhs, _char_mul=np.char.multiply, **kwargs):
+    return _char_mul(lhs, rhs)
+
+
+def _interp(arr, v, _Series=pd.Series, _charmod=np.char.mod):
+    """
+    Delegate to the most efficient string formatting technique based on
+    the length of the array.
+    """
+    if len(arr) >= 145:
+        return _Series(arr) % v
+
+    return _charmod(arr, v)
+
+
+@dispatch(Interp, np.ndarray)
+def compute_up(t, data, **kwargs):
+    if isinstance(t.lhs, Expr):
+        return _interp(data, t.rhs)
+    else:
+        return _interp(t.lhs, data)
+
+
+@compute_up.register(Interp, np.ndarray, (np.ndarray, base))
+@compute_up.register(Interp, base, np.ndarray)
+def compute_up_np_interp(t, lhs, rhs, **kwargs):
+    return _interp(lhs, rhs)
+
+
+
 @dispatch(BinOp, np.ndarray, (np.ndarray, base))
 def compute_up(t, lhs, rhs, **kwargs):
     return t.op(lhs, rhs)
@@ -311,3 +351,13 @@ def join_ndarray(expr, lhs, rhs, **kwargs):
     if isinstance(rhs, np.ndarray):
         rhs = DataFrame(rhs)
     return compute_up(expr, lhs, rhs, **kwargs)
+
+
+@dispatch(Coerce, np.ndarray)
+def compute_up(expr, data, **kwargs):
+    return data.astype(to_numpy_dtype(expr.schema))
+
+
+@dispatch(Concat, np.ndarray, np.ndarray)
+def compute_up(expr, lhs, rhs, _concat=np.concatenate, **kwargs):
+    return _concat((lhs, rhs), axis=expr.axis)
