@@ -14,29 +14,14 @@ from ..dispatch import dispatch
 
 from .core import compute, compute_up
 
+import py4j
+from pyspark import SparkContext
+import pyspark
+from pyspark.rdd import RDD
+
 __all__ = ['RDD', 'pyspark', 'SparkContext']
 
 
-class Dummy(object):
-    sum = max = min = count = distinct = mean = variance = stdev = None
-
-try:
-    import py4j
-    from pyspark import SparkContext
-    import pyspark
-    from pyspark.rdd import RDD
-    try:
-        from pyspark.sql import DataFrame as SparkDataFrame
-    except ImportError:
-        SparkDataFrame = Dummy
-    RDD.min
-except (AttributeError, ImportError):
-    SparkContext = Dummy
-    pyspark = Dummy()
-    pyspark.rdd = Dummy()
-    RDD = Dummy
-    SparkDataFrame = Dummy
-
 # PySpark adds a SIGCHLD signal handler, but that breaks other packages, so we
 # remove it
 # See https://issues.apache.org/jira/browse/SPARK-1394
@@ -138,13 +123,6 @@ def jgetattr(data, attr, default=None):
         return default
 
 
-@compute_up.register(Join, SparkDataFrame, SparkDataFrame)
-def spark_df_join(t, lhs, rhs, **kwargs):
-    # ship to rdd land, so we can reuse handling of combining records code
-    rdd = compute_up(t, lhs.rdd, rhs.rdd, **kwargs)
-    return lhs.sql_ctx.createDataFrame(rdd)
-
-
 @compute_up.register(Join, RDD, RDD)
 def spark_join(t, lhs, rhs, **kwargs):
     on_left = rowfunc(t.lhs[t.on_left])
