@@ -25,7 +25,7 @@ from copy import copy
 import sqlalchemy as sa
 
 from sqlalchemy import sql, Table, MetaData
-from sqlalchemy.sql import Selectable, Select
+from sqlalchemy.sql import Selectable, Select, functions as safuncs
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.sql.elements import ClauseElement, ColumnElement
 from sqlalchemy.engine import Engine
@@ -35,21 +35,27 @@ import toolz
 from toolz import unique, concat, pipe
 from toolz.curried import map
 
+import numpy as np
+import numbers
+
+import warnings
+
 from multipledispatch import MDNotImplementedError
 
-from odo.backends.sql import metadata_of_engine
+from odo.backends.sql import metadata_of_engine, dshape_to_alchemy
 
 from ..dispatch import dispatch
 
 from .core import compute_up, compute, base
 
-from ..expr import Projection, Selection, Field, Broadcast, Expr, IsIn
-from ..expr import (BinOp, UnaryOp, USub, Join, mean, var, std, Reduction,
-                    count, FloorDiv, UnaryStringFunction, strlen, DateTime)
+from ..expr import Projection, Selection, Field, Broadcast, Expr, IsIn, Slice
+from ..expr import (BinOp, UnaryOp, Join, mean, var, std, Reduction, count,
+                    FloorDiv, UnaryStringFunction, strlen, DateTime, Coerce)
 from ..expr import nunique, Distinct, By, Sort, Head, Label, ReLabel, Merge
-from ..expr import common_subexpression, Summary, Like, nelements
+from ..expr import common_subexpression, Summary, Like, nelements, Concat
 
 from ..expr.broadcast import broadcast_collect
+from ..expr.math import isnan
 
 from ..compatibility import reduce
 
@@ -87,7 +93,8 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(Broadcast, Select)
 def compute_up(t, s, **kwargs):
-    d = dict((t._scalars[0][c], list(inner_columns(s))[i])
+    cols = list(inner_columns(s))
+    d = dict((t._scalars[0][c], cols[i])
              for i, c in enumerate(t._scalars[0].fields))
     name = t._scalar_expr._name
     result = compute(t._scalar_expr, d, post_compute=False).label(name)
@@ -99,12 +106,24 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(Broadcast, Selectable)
 def compute_up(t, s, **kwargs):
-    d = dict((t._scalars[0][c], list(inner_columns(s))[i])
+    cols = list(inner_columns(s))
+    d = dict((t._scalars[0][c], cols[i])
              for i, c in enumerate(t._scalars[0].fields))
     name = t._scalar_expr._name
     return compute(t._scalar_expr, d, post_compute=False).label(name)
 
 
+@dispatch(Concat, (Select, Selectable), (Select, Selectable))
+def compute_up(t, lhs, rhs, **kwargs):
+    if t.axis != 0:
+        raise ValueError(
+            'Cannot concat along a non-zero axis in sql; perhaps you want'
+            " 'merge'?",
+        )
+
+    return select(lhs).union_all(select(rhs)).alias()
+
+
 @dispatch(Broadcast, sa.Column)
 def compute_up(t, s, **kwargs):
     expr = t._scalar_expr
@@ -139,15 +158,15 @@ def binop_sql(t, lhs, rhs, **kwargs):
     return sa.func.floor(lhs / rhs)
 
 
+@dispatch(isnan, ColumnElement)
+def compute_up(t, s, **kwargs):
+    return s == float('nan')
+
+
 @dispatch(UnaryOp, ColumnElement)
 def compute_up(t, s, **kwargs):
-    op = getattr(sa.func, t.symbol)
-    return op(s)
-
-
-@dispatch(USub, (sa.Column, Selectable))
-def compute_up(t, s, **kwargs):
-    return -s
+    sym = t.symbol
+    return getattr(t, 'op', getattr(safuncs, sym, getattr(sa.func, sym)))(s)
 
 
 @dispatch(Selection, Select)
@@ -237,8 +256,9 @@ def compute_up(t, lhs, rhs, **kwargs):
     if isinstance(rhs, ColumnElement):
         rhs = select(rhs)
     if name(lhs) == name(rhs):
-        lhs = lhs.alias('%s_left' % name(lhs))
-        rhs = rhs.alias('%s_right' % name(rhs))
+        left_suffix, right_suffix = t.suffixes
+        lhs = lhs.alias('%s%s' % (name(lhs), left_suffix))
+        rhs = rhs.alias('%s%s' % (name(rhs), right_suffix))
 
     lhs = alias_it(lhs)
     rhs = alias_it(rhs)
@@ -291,7 +311,10 @@ def compute_up(t, lhs, rhs, **kwargs):
     left_cols = cols(lhs)
     right_cols = cols(rhs)
 
-    fields = [f.replace('_left', '').replace('_right', '') for f in t.fields]
+    left_suffix, right_suffix = t.suffixes
+    fields = [
+        f.replace(left_suffix, '').replace(right_suffix, '') for f in t.fields
+    ]
     columns = [c for c in main_cols if c.name in t._on_left]
     columns += [c for c in left_cols
                 if c.name in fields and c.name not in t._on_left]
@@ -310,16 +333,14 @@ names = {
 
 
 @dispatch((nunique, Reduction), Select)
-def compute_up(t, s, **kwargs):
-    if t.axis != (0,):
+def compute_up(expr, data, **kwargs):
+    if expr.axis != (0,):
         raise ValueError('axis not equal to 0 not defined for SQL reductions')
-    d = dict((t._child[c], list(inner_columns(s))[i])
-             for i, c in enumerate(t._child.fields))
-    col = compute(t, d, post_compute=False)
-
-    s = copy(s)
-    s.append_column(col)
-    return s.with_only_columns([col])
+    data = data.cte(name=next(aliases))
+    cols = list(inner_columns(data))
+    d = dict((expr._child[c], cols[i])
+             for i, c in enumerate(expr._child.fields))
+    return select([compute(expr, d, post_compute=False)])
 
 
 @dispatch(Distinct, sa.Column)
@@ -500,24 +521,26 @@ def alias_it(s):
 
 @dispatch(By, Select)
 def compute_up(t, s, **kwargs):
+
+    # TODO: this should restrict according to dshape, not expression type
     if not (isinstance(t.grouper, (Field, Projection, DateTime)) or
             t.grouper is t._child):
-        raise ValueError("Grouper must be a projection, got %s" % t.grouper)
+        raise ValueError("Grouper must be a Field, Projection or DateTime, "
+                         "got %s of type %r" % (t.grouper,
+                                                type(t.grouper).__name__))
 
     s = alias_it(s)
 
-    if isinstance(t.apply, Reduction):
+    # TODO: Do we need to be this restrictive here?
+    if isinstance(t.apply, (Summary, Reduction)):
         reduction = compute(t.apply, {t._child: s}, post_compute=False)
+    else:
+        raise TypeError('apply must be a Summary or Reduction expression')
 
-    elif isinstance(t.apply, Summary):
-        reduction = compute(t.apply, {t._child: s}, post_compute=False)
-
-    grouper = [lower_column(s.c.get(col)) for col in t.grouper.fields]
-
-    grouper_columns = pipe(grouper, map(get_inner_columns), concat)
+    grouper = get_inner_columns(compute(t.grouper, {t._child: s}))
     reduction_columns = pipe(reduction.inner_columns, map(get_inner_columns),
                              concat)
-    columns = list(unique(chain(grouper_columns, reduction_columns)))
+    columns = list(unique(chain(grouper, reduction_columns)))
     if (not isinstance(s, sa.sql.selectable.Alias) or
             (hasattr(s, 'froms') and isinstance(s.froms[0],
                                                 sa.sql.selectable.Join))):
@@ -537,11 +560,19 @@ def compute_up(t, s, **kwargs):
                      order_by=get_clause(getattr(s, 'element', s), 'order_by'))
 
 
-@dispatch(Sort, ClauseElement)
+@dispatch(Sort, (Selectable, Select))
+def compute_up(t, s, **kwargs):
+    s = select(s.alias())
+    direction = sa.asc if t.ascending else sa.desc
+    cols = [direction(lower_column(s.c[c])) for c in listpack(t.key)]
+    return s.order_by(*cols)
+
+
+@dispatch(Sort, (sa.Table, ColumnElement))
 def compute_up(t, s, **kwargs):
     s = select(s)
     direction = sa.asc if t.ascending else sa.desc
-    cols = [direction(lower_column(getattr(s.c, c))) for c in listpack(t.key)]
+    cols = [direction(lower_column(s.c[c])) for c in listpack(t.key)]
     return s.order_by(*cols)
 
 
@@ -574,7 +605,10 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(sa.sql.FromClause)
 def get_inner_columns(sel):
-    return list(map(lower_column, sel.c.values()))
+    try:
+        return list(sel.inner_columns)
+    except AttributeError:
+        return list(map(lower_column, sel.c.values()))
 
 
 @dispatch(sa.sql.elements.ColumnElement)
@@ -591,11 +625,9 @@ def get_inner_columns(sel):
 
 @dispatch(sa.sql.functions.Function)
 def get_inner_columns(f):
-    assert len(f.clauses.clauses) == 1, \
-        'only single argument functions allowed'
     unique_columns = unique(concat(map(get_inner_columns, f.clauses)))
     lowered = [x.label(getattr(x, 'name', None)) for x in unique_columns]
-    return list(map(getattr(sa.func, f.name), lowered))
+    return [getattr(sa.func, f.name)(*lowered)]
 
 
 @dispatch(sa.sql.elements.Label)
@@ -727,6 +759,8 @@ def compute_up(expr, data, **kwargs):
 
 @toolz.memoize
 def table_of_metadata(metadata, name):
+    if metadata.schema is not None:
+        name = '.'.join((metadata.schema, name))
     if name not in metadata.tables:
         metadata.reflect(views=metadata.bind.dialect.supports_views)
     return metadata.tables[name]
@@ -744,6 +778,9 @@ def compute_up(expr, data, **kwargs):
 
 @dispatch(DateTime, (ClauseElement, sa.sql.elements.ColumnElement))
 def compute_up(expr, data, **kwargs):
+    if expr.attr == 'date':
+        return sa.func.date(data).label(expr._name)
+
     return sa.extract(expr.attr, data).label(expr._name)
 
 
@@ -788,3 +825,45 @@ def post_compute(_, s, **kwargs):
 @dispatch(IsIn, ColumnElement)
 def compute_up(expr, data, **kwargs):
     return data.in_(expr._keys)
+
+
+@dispatch(Slice, (Select, Selectable, ColumnElement))
+def compute_up(expr, data, **kwargs):
+    index = expr.index[0]  # [0] replace_slices returns tuple ((start, stop), )
+    if isinstance(index, slice):
+        start = index.start or 0
+        if start < 0:
+            raise ValueError('start value of slice cannot be negative'
+                             ' with a SQL backend')
+
+        stop = index.stop
+        if stop is not None and stop < 0:
+            raise ValueError('stop value of slice cannot be negative with a '
+                             'SQL backend.')
+
+        if index.step is not None and index.step != 1:
+            raise ValueError('step parameter in slice objects not supported '
+                             'with SQL backend')
+
+    elif isinstance(index, (np.integer, numbers.Integral)):
+        if index < 0:
+            raise ValueError('integer slice cannot be negative for the'
+                             ' SQL backend')
+        start = index
+        stop = start + 1
+    else:
+        raise TypeError('type %r not supported for slicing wih SQL backend'
+                        % type(index).__name__)
+
+    warnings.warn('The order of the result set from a Slice expression '
+                  'computed against the SQL backend is not deterministic.')
+
+    if stop is None:  # Represents open-ended slice. e.g. [3:]
+        return select(data).offset(start)
+    else:
+        return select(data).offset(start).limit(stop - start)
+
+
+@dispatch(Coerce, ColumnElement)
+def compute_up(expr, data, **kwargs):
+    return sa.cast(data, dshape_to_alchemy(expr.to)).label(expr._name)
