@@ -2,13 +2,14 @@ from __future__ import absolute_import, division, print_function
 
 import toolz
 import datashape
-import functools
-import keyword
+import re
+
+from keyword import iskeyword
+
 import numpy as np
 
-from toolz import concat, memoize, partial
+from toolz import concat, memoize, partial, first
 from toolz.curried import map, filter
-import re
 
 from datashape import dshape, DataShape, Record, Var, Mono, Fixed
 from datashape.predicates import isscalar, iscollection, isboolean, isrecord
@@ -17,17 +18,19 @@ from ..compatibility import _strtypes, builtins, boundmethod
 from .core import Node, subs, common_subexpression, path
 from .method_dispatch import select_functions
 from ..dispatch import dispatch
+from .utils import hashable_index, replace_slices
+
 
 __all__ = ['Expr', 'ElemWise', 'Field', 'Symbol', 'discover', 'Projection',
            'projection', 'Selection', 'selection', 'Label', 'label', 'Map',
-           'ReLabel', 'relabel', 'Apply', 'Slice', 'shape', 'ndim', 'label',
-           'symbol']
+           'ReLabel', 'relabel', 'Apply', 'apply', 'Slice', 'shape', 'ndim',
+           'label', 'symbol', 'Coerce', 'coerce']
 
 
 _attr_cache = dict()
 
 
-def isvalid_identifier(s, regex=re.compile('^[_a-zA-Z][_a-zA-Z0-9]*$')):
+def isvalid_identifier(s):
     """Check whether a string is a valid Python identifier
 
     Examples
@@ -47,7 +50,9 @@ def isvalid_identifier(s, regex=re.compile('^[_a-zA-Z][_a-zA-Z0-9]*$')):
     >>> isvalid_identifier(None)
     False
     """
-    return not not s and not keyword.iskeyword(s) and regex.match(s) is not None
+    # the re module compiles and caches regexs so no need to compile it
+    return (s is not None and not iskeyword(s) and
+            re.match(r'^[_a-zA-Z][_a-zA-Z0-9]*$', s) is not None)
 
 
 def valid_identifier(s):
@@ -236,8 +241,8 @@ class Symbol(Expr):
     """
     Symbolic data.  The leaf of a Blaze expression
 
-    Example
-    -------
+    Examples
+    --------
     >>> points = symbol('points', '5 * 3 * {x: int, y: int}')
     >>> points
     points
@@ -286,25 +291,28 @@ class ElemWise(Expr):
 
 class Field(ElemWise):
     """
-    A single field from an expression
+    A single field from an expression.
 
-    Get a single field from an expression with record-type schema.  Collapses
-    that record.  We store the name of the field in the ``_name`` attribute.
-
-    SELECT a
-    FROM table
+    Get a single field from an expression with record-type schema.
+    We store the name of the field in the ``_name`` attribute.
 
+    Examples
+    --------
     >>> points = symbol('points', '5 * 3 * {x: int32, y: int32}')
     >>> points.x.dshape
     dshape("5 * 3 * int32")
+
+    For fields that aren't valid Python identifiers, use ``[]`` syntax:
+
+    >>> points = symbol('points', '5 * 3 * {"space station": float64}')
+    >>> points['space station'].dshape
+    dshape("5 * 3 * float64")
     """
     __slots__ = '_hash', '_child', '_name'
 
     def __str__(self):
-        if re.match('^\w+$', self._name):
-            return '%s.%s' % (self._child, self._name)
-        else:
-            return "%s['%s']" % (self._child, self._name)
+        fmt = '%s.%s' if isvalid_identifier(self._name) else '%s[%r]'
+        return fmt % (self._child, self._name)
 
     @property
     def _expr(self):
@@ -321,10 +329,7 @@ class Field(ElemWise):
 
 
 class Projection(ElemWise):
-    """ Select fields from data
-
-    SELECT a, b, c
-    FROM table
+    """Select a subset of fields from data.
 
     Examples
     --------
@@ -332,7 +337,6 @@ class Projection(ElemWise):
     ...                   'var * {name: string, amount: int, id: int}')
     >>> accounts[['name', 'amount']].schema
     dshape("{name: string, amount: int32}")
-
     >>> accounts[['name', 'amount']]
     accounts[['name', 'amount']]
 
@@ -379,9 +383,6 @@ def projection(expr, names):
 projection.__doc__ = Projection.__doc__
 
 
-from .utils import hashable_index, replace_slices
-
-
 def sanitize_index_lists(ind):
     """ Handle lists/arrays of integers/bools as indexes
 
@@ -412,6 +413,18 @@ def sliceit(child, index):
 
 
 class Slice(Expr):
+    """Elements `start` until `stop`. On many backends, a `step` parameter
+    is also allowed.
+
+    Examples
+    --------
+    >>> from blaze import symbol
+    >>> accounts = symbol('accounts', 'var * {name: string, amount: int}')
+    >>> accounts[2:7].dshape
+    dshape("5 * {name: string, amount: int32}")
+    >>> accounts[2:7:2].dshape
+    dshape("3 * {name: string, amount: int32}")
+    """
     __slots__ = '_hash', '_child', '_index'
 
     @property
@@ -423,10 +436,11 @@ class Slice(Expr):
         return replace_slices(self._index)
 
     def __str__(self):
-        if type(self.index) == tuple:
-            return '%s[%s]' % (self._child, ', '.join(map(str, self._index)))
+        if isinstance(self.index, tuple):
+            index = ', '.join(map(str, self._index))
         else:
-            return '%s[%s]' % (self._child, self._index)
+            index = str(self._index)
+        return '%s[%s]' % (self._child, index)
 
 
 class Selection(Expr):
@@ -474,7 +488,7 @@ selection.__doc__ = Selection.__doc__
 
 
 class Label(ElemWise):
-    """A Labeled expression
+    """An expression with a name.
 
     Examples
     --------
@@ -502,11 +516,10 @@ class Label(ElemWise):
     def _get_field(self, key):
         if key[0] == self.fields[0]:
             return self
-        else:
-            raise ValueError("Column Mismatch: %s" % key)
+        raise ValueError("Column Mismatch: %s" % key)
 
     def __str__(self):
-        return "label(%s, %r)" % (self._child, self.label)
+        return 'label(%s, %r)' % (self._child, self.label)
 
 
 def label(expr, lab):
@@ -533,6 +546,27 @@ class ReLabel(ElemWise):
     Traceback (most recent call last):
         ...
     ValueError: Cannot relabel non-existent child fields: {'not_a_column'}
+    >>> s = symbol('s', 'var * {"0": int64}')
+    >>> s.relabel({'0': 'foo'})
+    s.relabel({'0': 'foo'})
+    >>> s.relabel(0='foo') # doctest: +SKIP
+    Traceback (most recent call last):
+        ...
+    SyntaxError: keyword can't be an expression
+
+    Notes
+    -----
+    When names are not valid Python names, such as integers or string with
+    spaces, you must pass a dictionary to ``relabel``. For example
+
+    .. code-block:: python
+
+       >>> s = symbol('s', 'var * {"0": int64}')
+       >>> s.relabel({'0': 'foo'})
+       s.relabel({'0': 'foo'})
+       >>> t = symbol('t', 'var * {"whoo hoo": ?float32}')
+       >>> t.relabel({"whoo hoo": 'foo'})
+       t.relabel({'whoo hoo': 'foo'})
 
     See Also
     --------
@@ -548,8 +582,12 @@ class ReLabel(ElemWise):
                                  for name, dtype in param]))
 
     def __str__(self):
-        return ('%s.relabel(%s)' %
-                (self._child, ', '.join('%s=%r' % l for l in self.labels)))
+        labels = self.labels
+        if all(map(isvalid_identifier, map(first, labels))):
+            rest = ', '.join('%s=%r' % l for l in labels)
+        else:
+            rest = '{%s}' % ', '.join('%r: %r' % l for l in labels)
+        return '%s.relabel(%s)' % (self._child, rest)
 
 
 def relabel(child, labels=None, **kwargs):
@@ -641,7 +679,7 @@ class Apply(Expr):
 
     You must provide the datashape of the result with the ``dshape=`` keyword.
     For datashape examples see
-        http://datashape.pydata.org/grammar.html#some-simple-examples
+    http://datashape.pydata.org/grammar.html#some-simple-examples
 
     If using a chunking backend and your operation may be safely split and
     concatenated then add the ``splittable=True`` keyword argument
@@ -667,6 +705,33 @@ class Apply(Expr):
         return dshape(self._dshape)
 
 
+class Coerce(Expr):
+    """Coerce an expression to a different type.
+
+    Examples
+    --------
+    >>> t = symbol('t', '100 * float64')
+    >>> t.coerce(to='int64')
+    t.coerce(to='int64')
+    >>> t.coerce('float32')
+    t.coerce(to='float32')
+    >>> t.coerce('int8').dshape
+    dshape("100 * int8")
+    """
+    __slots__ = '_hash', '_child', 'to'
+
+    @property
+    def schema(self):
+        return self.to
+
+    @property
+    def dshape(self):
+        return DataShape(*(self._child.shape + (self.schema,)))
+
+    def __str__(self):
+        return '%s.coerce(to=%r)' % (self._child, str(self.schema))
+
+
 def apply(expr, func, dshape, splittable=False):
     return Apply(expr, func, datashape.dshape(dshape), splittable)
 
@@ -719,13 +784,21 @@ def ndim(expr):
     return len(shape(expr))
 
 
+def coerce(expr, to):
+    return Coerce(expr, dshape(to) if isinstance(to, _strtypes) else to)
+
+
+coerce.__doc__ = Coerce.__doc__
+
+
 dshape_method_list.extend([
     (lambda ds: True, set([apply])),
     (iscollection, set([shape, ndim])),
+    (lambda ds: iscollection(ds) and isscalar(ds.measure), set([coerce]))
 ])
 
 schema_method_list.extend([
-    (isscalar, set([label, relabel])),
+    (isscalar, set([label, relabel, coerce])),
     (isrecord, set([relabel])),
 ])
 
